@@ -17,7 +17,11 @@ import torch.nn.functional as F
 from torch.autograd import grad
 from torch.optim import Optimizer
 
-from .scaler import tweakBAF
+
+if __name__ == "__main__":
+    from shared import *
+else:
+    from .shared import *
 
 
 #np.random.seed(0)
@@ -1123,7 +1127,7 @@ def modelCNAgenerator(CNAfull, chr, start1, end1, model, Ncall, info, returnReg=
     #CNA_mean0 = np.mean(np.sum(CNAfull, axis=2), axis=1) / 2
     CNA_mean0 = np.median(np.sum(CNAfull, axis=2), axis=1) / 2
 
-    #TODO!: !I think I was doing this poorly with mean!
+    
     
     CNA_mean = np.round(CNA_mean0) - 1
     argDouble = np.argwhere(CNA_mean >= 1)[:, 0]
@@ -1834,9 +1838,21 @@ def subset_calculateError(pred_RDR, pred_BAF, RDR, HAP, chr, boolSubset, noiseRD
             #BAF_error = minimalError - BAF_error
             #BAF_error = torch.sum(BAF_error, axis=1)
             
-            BAF_int = HAP[subset1, :, 1].float() / (torch.sum(HAP[subset1], axis=2).float() + 1e-5)
 
-            BAF_error = torch.sum(  func1((pred_BAF1 - BAF_int) * noiseBAFInverse[subset1])  , axis=1) + 1e-5
+            if True:
+                BAF_int = HAP[subset1, :, 1].float() / (torch.sum(HAP[subset1], axis=2).float() + 1e-5)
+                BAF_error = torch.sum(  func1((pred_BAF1 - BAF_int) * noiseBAFInverse[subset1])  , axis=1) + 1e-5
+            else: # Old method prior to FEB 7 2024 . TODO potentionally remove the unneeded code. 
+                #std2 = std2 + (0.25 / weight_sum)
+                weight_sum = 0.25 / noiseBAFInverse[subset1]
+                weight_sum_rel = weight_sum / (torch.sum(HAP[subset1], axis=2) + 1e-5)
+                weight_sum_rel[weight_sum_rel>1] = 1
+                BAF_error = ( torch.log(pred_BAF1) * HAP[subset1, :, 1] * weight_sum_rel  ) + ( torch.log(1 - pred_BAF1) * HAP[subset1, :, 0] * weight_sum_rel  )
+
+            
+                BAF_error = torch.sum(BAF_error, axis=1)
+                
+
             
 
             #print ( type(BAF_error))
@@ -2267,7 +2283,7 @@ def measurementCalculator(savedCNA, model, doBAF):
     mean1 = torch.mean(predRDR, axis=lastDim-1)
 
 
-    if False: #TODO: REEMOVE! TEMP UNDO! 
+    if False: #TODO: delete not needed code. 
         biasAdjustment = model.biasAdjuster() + 1
 
 
@@ -2312,6 +2328,174 @@ def measurementCalculator(savedCNA, model, doBAF):
 
 
 
+def OLD_CNAbulkEst(bestCNA, RDR, HAP):
+
+    inverse1 = uniqueProfileMaker(bestCNA)
+
+    _, index1, count1 = np.unique(inverse1, return_index=True, return_counts=True)
+
+    index1 = index1[count1 >= 5]
+
+    CNA_new = np.zeros( (index1.shape[0], bestCNA.shape[1], bestCNA.shape[2] ) , dtype=int)
+
+    for a in range(index1.shape[0]):
+        args1 = np.argwhere(inverse1 == index1[a])[:, 0]
+
+        RDR_now, HAP_now = RDR[args1], HAP[args1]
+        RDR_now = np.mean(RDR_now, axis=0)
+        HAP_now = np.sum(HAP_now, axis=0)
+        BAF_now = HAP_now[:, 1] / (np.sum(HAP_now, axis=1) + 1e-5)
+
+        ploidy1 = np.mean(bestCNA[index1[a]]) * 2
+        RDR_now = RDR_now * ploidy1
+        totalNow = np.round(RDR_now).astype(int)
+
+        Bcopy = np.round(totalNow * BAF_now)
+
+        CNA1 = np.array( [totalNow - Bcopy, Bcopy]  ).T
+
+        CNA_new[a] = CNA1
+    
+
+    print ('CNA_new')
+    print (CNA_new.shape)
+
+    CNA_both = np.concatenate((bestCNA, CNA_new), axis=0)
+
+    inverse2 = uniqueProfileMaker(CNA_both)
+
+    argGood = np.argwhere(np.isin( inverse2, inverse2[:bestCNA.shape[0]] ) == False)[:, 0]
+
+    CNA_new = CNA_both[argGood]
+
+    print (CNA_new.shape)
+    
+    #quit()
+
+    return CNA_new
+
+
+
+
+def CNAbulkEst(bestCNA, RDR, HAP):
+
+    
+    
+    #CNA_new = np.zeros( bestCNA.shape , dtype=int)
+    CNA_new = np.copy(bestCNA)
+
+    #plt.imshow(  CNA_new[0::20, :, 1] / ( np.sum(CNA_new[0::20], axis=2) + 1e-5 )  , cmap='bwr' )
+    #plt.show()   
+
+    #haplotypePlotter(CNA_new, doCluster=False)
+
+    for a in range(bestCNA.shape[1]):
+
+        inverse1 = uniqueValMaker(bestCNA[:, a])
+        unique1, counts1 = np.unique(inverse1, return_counts=True)
+
+
+        for b in range(unique1.shape[0]):
+            args1 = np.argwhere(inverse1 == unique1[b])[:, 0]
+            if args1.shape[0] > 10:
+
+                copy1 = bestCNA[args1[0], a]
+                copy_total = float(np.sum(copy1))
+
+                B_options = np.arange(copy_total+1)
+                A_options = copy_total - B_options
+
+                HAP_now = np.sum(HAP[args1, a], axis=0)
+                #BAF_now = HAP_now[1] / (np.sum(HAP_now) + 1e-5)
+                #B_copy = np.round(copy_total * BAF_now)
+                #A_copy = copy_total - B_copy
+
+                
+
+                #print (BAF_now)
+
+                lossList = np.zeros(A_options.shape[0])
+                for c in range(A_options.shape[0]):
+                    BAF1 = B_options[c] / (A_options[c] + B_options[c] + 1e-5)
+                    BAF1 = (BAF1 * 0.9) + 0.05
+                    logA = np.log(1 - BAF1)
+                    logB = np.log(BAF1)
+                    loss1 = (logA * HAP_now[0]) + (logB * HAP_now[1])
+                    lossList[c] = loss1
+                argBest = np.argmax(lossList)
+                A_copy, B_copy = A_options[argBest], B_options[argBest]
+
+
+                good1 = True
+
+                if False:#copy_total == 1: #TODO just an attempt
+                    if np.sum(HAP_now) > 100:
+                        if np.min(HAP_now) / np.sum(HAP_now) > 0.3:
+                            A_copy, B_copy = 1, 1
+
+                            print ("Triggered")
+                            print (copy1, A_copy, B_copy)
+                            print (HAP_now)
+
+
+                #if a == bestCNA.shape[1] - 1:
+                #    print ('A')
+                #    print (copy1, A_copy, B_copy)
+                #    print (HAP_now)
+
+                if 0 in [A_copy, B_copy]:
+
+                    if np.sum(HAP_now) < 100:
+                        good1 = False
+                    
+                    #print ("copy")
+                    #print (copy1, A_copy, B_copy)
+                    #print (HAP_now)
+
+                    weird1 = False 
+                    if (copy1[0] != 0) and (A_copy == 0):
+                        weird1 = True
+                    if (copy1[1] != 0) and (B_copy == 0):
+                        weird1 = True
+
+                    #if weird1:
+                    #    print ("copy")
+                    #    print (copy1, A_copy, B_copy)
+                    #    print (HAP_now)
+
+
+                if good1:
+                    CNA_new[args1, a, 0] = A_copy
+                    CNA_new[args1, a, 1] = B_copy
+
+
+    
+    
+    #print ('CNA_new')
+    #print (CNA_new.shape)
+                
+    #plt.imshow(  CNA_new[0::20, :, 1] / ( np.sum(CNA_new[0::20], axis=2) + 1e-5 )  , cmap='bwr' )
+    #plt.show()    
+
+    if False:
+        CNA_both = np.concatenate((bestCNA, CNA_new), axis=0)
+
+        inverse2 = uniqueProfileMaker(CNA_both)
+
+        argGood = np.argwhere(np.isin( inverse2, inverse2[:bestCNA.shape[0]] ) == False)[:, 0]
+
+        CNA_new = CNA_both[argGood]
+    
+    #print (CNA_new.shape)
+    
+    #haplotypePlotter(CNA_new, doCluster=False)
+
+    
+    #quit()
+
+    return CNA_new
+
+
 
 def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, Ncall, noiseRDR, noiseBAF, withAdjust, balance, doDouble=False, stopIter=False):
 
@@ -2380,7 +2564,7 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
     #optimizer = torch.optim.Adam(model.parameters(), lr = learningRate, betas=(0.8, 0.99))
     #optimizer = torch.optim.RMSprop(model.parameters(), lr = learningRate, alpha=(0.9))
 
-    #optimizer = torch.optim.SGD(model.parameters(), lr = learningRate) #TODO REMOVE!!! Trying for memory issue! 
+    #optimizer = torch.optim.SGD(model.parameters(), lr = learningRate) #TODO delete unneeded. Tried for memory issue. 
 
 
 
@@ -2414,11 +2598,13 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
     while continue1:
         iter += 1
 
+        print ('iterations', iter)
+
         #gapLearn = 2 #Default 2
         gapLearn = 2 #Default 2
         gapTime = 10#5 #Default 5
 
-        if len(cellProbList) > gapTime:
+        if False:#len(cellProbList) > gapTime:
             cellProb_array = np.array(cellProbList)
             cellProb_max1 = np.max(cellProb_array[:-gapTime])
             cellProb_max2 = np.max(cellProb_array[-gapTime:])
@@ -2430,12 +2616,12 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
 
         if stopIter:
             #if iter == 100:
-            if iter == 200:
+            if iter == 400:
                 continue1 = False
 
 
         if continue1:
-            print (iter)
+            #print (iter)
             info = [iter]
 
 
@@ -2511,6 +2697,9 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
                 argWeight = np.argwhere(weight1 > (0.5 / RDR.shape[0]))
                 CNAfull_best = savedCNA[savedCNA_mapper[argWeight[:, 0], argWeight[:, 1]]]
 
+                bestCNA = savedCNA[savedCNA_mapper[ multPos[bestFit, 0], multPos[bestFit, 1] ] ]
+                CNA_new = CNAbulkEst(bestCNA, RDR, HAP)
+
 
                 CNA_randoms = np.copy(CNAfull_best)
                 chrRand = np.random.randint(22, size=CNA_randoms.shape[0])
@@ -2534,13 +2723,14 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
                 inverse_temp = uniqueValMaker( CNAfull_temp.reshape((CNAfull_temp.shape[0],  CNAfull_temp.shape[1]*CNAfull_temp.shape[2] )) )
                 _, index_temp = np.unique(inverse_temp, return_index=True)
 
-                print ("A")
-                print (np.argwhere(index_temp >=  CNAfull.shape[0] ).shape)
-                print (np.argwhere(index_temp <  CNAfull.shape[0] ).shape)
+                #print ("A")
+                #print (np.argwhere(index_temp >=  CNAfull.shape[0] ).shape)
+                #print (np.argwhere(index_temp <  CNAfull.shape[0] ).shape)
 
                 #quit()
                 #CNAfull_now = np.concatenate((CNAfull, CNAfull_best), axis=0)
-                CNAfull_now = np.concatenate((CNAfull, CNAfull_best, CNA_randoms), axis=0)
+                #CNAfull_now = np.concatenate((CNAfull, CNAfull_best, CNA_randoms), axis=0)
+                CNAfull_now = np.concatenate((CNAfull, CNAfull_best, CNA_randoms, CNA_new), axis=0)
 
                 inverse1 = uniqueValMaker( CNAfull_now.reshape((CNAfull_now.shape[0],  CNAfull_now.shape[1]*CNAfull_now.shape[2] )) )
                 _, index1 = np.unique(inverse1, return_index=True)
@@ -2610,9 +2800,9 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
 
             
 
-            print ('')
+            #print ('')
             #print (cellProb)
-            print (torch.mean(cellProb))
+            #print (torch.mean(cellProb))
             #print ('cell', torch.max(cellProb))
             #print (np.mean(modelProbSum.data.numpy() ))
             cellProb_mean = torch.mean(cellProb).data.numpy()
@@ -2629,25 +2819,6 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
 
             if True:
 
-                #print (RDR.shape)
-                #print (multPaste.shape)
-
-                #plt.imshow(multPaste)
-                #plt.show()
-
-                #bestFit = np.argmax(multPaste, axis=0)
-
-
-
-
-                #counterAll[np.unique(multPos[bestFit, 1])] = counterAll[np.unique(multPos[bestFit, 1])] + 1
-
-                #if iter == iterPass:
-                #    CNAfull = CNAfull[np.unique(multPos[bestFit, 1])]
-                #    CNAfull_now = CNAfull_now[np.unique(multPos[bestFit, 1])]
-
-                
-
                 print ('bestFit', np.unique(bestFit).shape, bestFit.shape)
 
                 bestCNA = savedCNA[savedCNA_mapper[ multPos[bestFit, 0], multPos[bestFit, 1] ] ]
@@ -2658,12 +2829,12 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
 
                 diff1 = bestCNA[:, 1:, 0] - bestCNA[:, :-1, 0]
                 diff1[diff1!=0] = 1
-                print ('diff1', np.sum(diff1))
+                #print ('diff1', np.sum(diff1))
 
                 np.savez_compressed(predict_file, bestCNA)
 
 
-                print ('counterAll', np.argwhere( counterAll >= 1).shape, np.argwhere( counterAll >= (iter // 2) + 1  ).shape ,iter,  (iter // 2) + 1 )
+                #print ('counterAll', np.argwhere( counterAll >= 1).shape, np.argwhere( counterAll >= (iter // 2) + 1  ).shape ,iter,  (iter // 2) + 1 )
 
 
 
@@ -2677,17 +2848,17 @@ def trainModel(CNAfull, chr, RDR, HAP, originalError, modelName, predict_file, N
             #    print("{:>30}: {:>8}".format(name, sizeof_fmt(size)))
 
             
-            print ("A")
+            #print ("A")
             optimizer.zero_grad()
-            print ("B")
+            #print ("B")
             loss.backward()
-            print ("C")
+            #print ("C")
             #if withAdjust:
             #    print ("C1")
             #    lossAdjustment.backward()
-            print ('C2')
+            #print ('C2')
             optimizer.step()
-            print ("D")
+            #print ("D")
 
 
 
@@ -2868,7 +3039,7 @@ def findBestCNA(CNAfull, chr, RDR, BAF, originalError, modelName, bestCNAFile, N
 
 
 
-def simpleTrain(RDR_file, HAP_file, chr_file, initialCNA_file, initialUniqueCNA_file, originalError_file, modelName, predict_file, Ncall, noise_file, BAF_noise_file, balance, withAdjust, stopIter=True):
+def simpleTrain(RDR_file, HAP_file, chr_file, initialCNA_file, initialUniqueCNA_file, originalError_file, modelName, predict_file, Ncall, noise_file, BAF_noise_file, balance, withAdjust, stopIter=False):
 
 
     
@@ -2930,7 +3101,7 @@ def simplePredict(RDR_file, BAF_file, chr_file, initialCNA_file, initialUniqueCN
         BAF = ''
     else:
         BAF = loadnpz(BAF_file)
-    chr = loadnpz(chr_file) - 1 #TODO REMOVE May 22 2023
+    chr = loadnpz(chr_file) - 1 
 
 
 
@@ -2971,8 +3142,6 @@ def simplePredict(RDR_file, BAF_file, chr_file, initialCNA_file, initialUniqueCN
 
 
 
-
-
 def easyRunRL(outLoc):
 
 
@@ -3002,11 +3171,4 @@ def easyRunRL(outLoc):
     #balance = 2.0
     balance = 1.0
 
-    simpleTrain(RDR_file, HAP_file, chr_file, initialCNA_file, initialUniqueCNA_file, originalError_file, modelName, predict_file, Ncall, noise_file, BAF_noise_file, balance, withAdjust)
-
-
-#trainModel()
-#quit()
-
-#Idea: Maybe don't allow amplification from 1 to 2 (for example) if the level of all bins is 3 in the region.
-#If it under-amplifies for all bins, it may be a stupid sample.
\ No newline at end of file
+    simpleTrain(RDR_file, HAP_file, chr_file, initialCNA_file, initialUniqueCNA_file, originalError_file, modelName, predict_file, Ncall, noise_file, BAF_noise_file, balance, withAdjust, stopIter=True)
