@@ -3,7 +3,7 @@
 import numpy as np
 
 
-import matplotlib.pyplot as plt
+#import matplotlib.pyplot as plt
 import time
 import scipy
 from scipy import stats
@@ -23,16 +23,13 @@ if __name__ == "__main__":
 else:
     from .shared import *
 
+from tqdm import tqdm
 
 
 
 
 
 
-predCNA = loadnpz('/Users/stefanivanovic/Desktop/Coding/Bio/CNA/data/TN3_example/model/pred_now.npz')
-haplotypePlotter(predCNA, doCluster=True)
-quit()
-
 
 
 def estimateHapNoise(HAP):
@@ -445,7 +442,7 @@ def findVariableBins(RDR_file, bins_file, chr_file, totalRead_file, doBAF, BAF_f
     for a in range(unique1.shape[0]):
         #for a in range(1):
 
-        print (a)
+        #print (a)
 
 
         args1 = np.argwhere(chr == unique1[a])[:, 0]
@@ -465,8 +462,8 @@ def findVariableBins(RDR_file, bins_file, chr_file, totalRead_file, doBAF, BAF_f
         #print (RDR_mini.shape)
         #quit()
 
-        print (RDR_mini.shape)
-        print (BAF_mini.shape)
+        #print (RDR_mini.shape)
+        #print (BAF_mini.shape)
         
         bins_mini = coreFindBins(RDR_mini, doBAF, BAF_mini)
 
@@ -474,7 +471,7 @@ def findVariableBins(RDR_file, bins_file, chr_file, totalRead_file, doBAF, BAF_f
 
 
         _, index1 = np.unique(bins_mini, return_index=True)
-        print (index1)
+        #print (index1)
 
 
 
@@ -631,8 +628,8 @@ def applyVariableBins(RDR_file, bins_file, chr_file, RDR_file2, noise_file, chr_
 
         HAP_sum[:, count1] = np.sum(HAP[:, args1], axis=1)
         
-        #data_avg[:, count1] = np.mean(data[:, args1], axis=1)
-        data_avg[:, count1] = np.median(data[:, args1], axis=1)
+        data_avg[:, count1] = np.mean(data[:, args1], axis=1)
+        #data_avg[:, count1] = np.median(data[:, args1], axis=1)
 
         #adjust_avg[count1] = np.mean(adjustment[args1])
 
@@ -654,25 +651,6 @@ def applyVariableBins(RDR_file, bins_file, chr_file, RDR_file2, noise_file, chr_
         
         noise_mini = data[:, args1] - data_avg[:, count1].reshape((-1, 1))
 
-        '''
-        noise_mini_fft = np.fft.fft(noise_mini, axis=1) 
-        noise_mini_fft = noise_mini_fft / (noise_mini.shape[1] ** 0.5) #This just corrects for the scaling included in np.fft.fft
-        noise_mini_fft = np.abs(noise_mini_fft)[:, 1:] ** 2
-        noise_mini_fft = noise_mini_fft / noise_mini.shape[1] #To make it an average not a sum
-
-        
-
-        arange1 = np.arange(noise_mini_fft.shape[1]) + 1
-        arange2 = np.min(  np.array([arange1, arange1[-1::-1] ]), axis=0 )
-        arange3 = arange2
-        
-
-        arange3 = arange3.reshape((1, -1))
-        
-
-        noise_mini_val = noise_mini_fft / arange3 #(arange2 ** 2) 
-        noise_mini_val = np.sum(noise_mini_val, axis=1) ** 0.5
-        '''
         
         noise_mini_val = estimateRDRnoise(noise_mini) ** 0.5
 
@@ -684,10 +662,7 @@ def applyVariableBins(RDR_file, bins_file, chr_file, RDR_file2, noise_file, chr_
 
 
 
-    #diff1 = np.abs(data_avg[:, 1:] - data_avg[:, :-1])
-    #diff1 = np.mean(diff1, axis=1)
-    #noise1 = noise1 / np.mean(noise1, axis=1).reshape((-1, 1))
-    #noise1 = noise1 * diff1.reshape((-1, 1))
+    
 
     for a in range(data.shape[0]):
         noise1[a] = noise1[a] / np.mean(data_avg[a])
@@ -698,39 +673,18 @@ def applyVariableBins(RDR_file, bins_file, chr_file, RDR_file2, noise_file, chr_
     np.savez_compressed(chr_file2, chr_avg)
     np.savez_compressed(noise_file, noise1)
 
-    #np.savez_compressed(adjustment_file2, adjust_avg)
-
-    
 
     if doBAF:
         np.savez_compressed(BAF_file2, HAP_sum)
         np.savez_compressed(BAF_noise_file, BAF_noise)
         
 
-    #np.savez_compressed(call_file2, call_avg)
-
-
-
-
-
-#RDR_file = './data/DLP/filtered_RDR.npz'
-#chr_file = './data/DLP/chr.npz'
-#bins_file = './data/DLP/bins.npz'
-#RDR_file2 = './data/DLP/filtered_RDR_avg2.npz'
-#noise_file = './data/DLP/filtered_RDR_noise2.npz'
-#chr_file2 = './data/DLP/chr_avg2.npz'
-#call_file = './fromServer/DLP_calls.npz'
-#call_file2 = './data/DLP/calls_avg2.npz'
 
 
 
 
 
 
-#folder1 = 'DLP'
-#folder1 = '10x'
-#folder1 = 'ACT10x'
-folder1 = 'TN3'
 
 bins_file = './data/' + folder1 + '/binScale/bins.npz'
 chr_file = './data/' + folder1 + '/initial/chr_100k.npz'
@@ -742,12 +696,6 @@ doBAF = True
 BAF_file = './data/' + folder1 + '/initial/HAP_100k.npz'
 BAF_file2 = './data/' + folder1 + '/binScale/filtered_HAP_avg.npz'
 BAF_noise_file = './data/' + folder1 + '/binScale/BAF_noise.npz'
-#adjustment_file = './data/' + folder1 + '/initial/gc_adjustment_100k.npz'
-#adjustment_file2 = './data/' + folder1 + '/initial/gc_adjustment_avg.npz'
-
-#BAF_noise_file = './data/' + folder1 + '/binScale/filtered_BAF_noise.npz'
-
-
 
 #applyVariableBins(RDR_file, bins_file, chr_file, RDR_file2, noise_file, chr_file2, doBAF, BAF_file=BAF_file, BAF_file2=BAF_file2, BAF_noise_file=BAF_noise_file)
 #quit()
@@ -770,13 +718,7 @@ def mapBAF(x):
 #print (RDR.shape)
 #quit()
 
-def tweakBAF(x):
 
-    delta = 0.01
-
-    x = (x * (1.0 - (delta * 2) ) ) + delta
-
-    return x
 
 
 
@@ -794,21 +736,17 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
     def findBestRegion(RDR, HAP):#, N):
 
-        #print (RDR_change, BAF_change)
-        #quit()
+
 
         RDR = RDR - np.mean(RDR)
         RDR_cumsum = paddedCumSum(RDR)
         RDR_sq_cumsum = paddedCumSum(RDR**2)
 
-        #if BAF != '':
-        #    BAF = BAF - np.mean(BAF)
-        #    BAF_cumsum = paddedCumSum(BAF)
-        #    BAF_sq_cumsum = paddedCumSum(BAF**2)
+       
 
         if type(HAP) != type(''):
             BAF = HAP[:, 1] / (np.sum(HAP, axis=1) + 1e-5) #Just removing division by zero
-            weight = np.sum(HAP, axis=1)#.astype(float)
+            weight = np.sum(HAP, axis=1)
             
             
 
@@ -820,13 +758,10 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
         size1 = RDR.shape[0]
 
-        #N = 100
+
         N = 1
-        #print ('findBestRegion')
-        #print (N)
-        #print (RDR.shape)
         sizeRound = ((RDR.shape[0] - 1) // N) + 1
-        #print (sizeRound)
+        
 
         data1 = np.zeros((  sizeRound ** 2, 3))
         count1 = 0
@@ -836,33 +771,16 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
                 b = (b0 * N)
 
 
-                #if b >= (a + 5):
                 if b >= (a + 5):
-                    #continue1 = False
-                    #if RDR.shape[0] > 10:
-                    #    if (a % 2 == 0) and (b % 2 == 1):
-                    #        continue1 = True
-                    #else:
-                    #    continue1 = True
+                    
 
-                    if True:#continue1:
-                        #RDR1 = RDR[a:b]
-
-                        #if BAF == '':
-                        #    BAF1 = ''
-                        #else:
-                        #    BAF1 = BAF[a:b]
-                        #    weight1 = weight[a:b]
-
-                        #'''
-                        #error1 = giveRegionError(RDR1, BAF1, a, b, size1)
+                    if True:
 
                         length0 = b - a
                         error1_sq = (RDR_sq_cumsum[b] - RDR_sq_cumsum[a]) / length0
                         error1_me = (RDR_cumsum[b] - RDR_cumsum[a]) / length0
 
                         error1 = error1_sq - (error1_me ** 2)
-                        #error1 = (error1 / (RDR_change ** 2)) + (5 / length0)
 
                         error1 = (error1 / length0) ** 0.5
 
@@ -880,59 +798,24 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
                                 error2_sq = (BAF_sq_cumsum[b] - BAF_sq_cumsum[a]) / (weight_sum ** 2)
                                 error2_me = (BAF_cumsum[b] - BAF_cumsum[a]) / weight_sum
 
-                                #error2 = error2_sq - (error2_me ** 2)
+
                                 error2 = error2_sq - ((error2_me ** 2) * weight_sq / (  weight_sum ** 2  ) )
 
+                                if error2 < 0:
+                                    error2 = 0
+
 
                                 error2 = error2 + (0.25 / (weight_sum) ) #Adding the intrinsic noise in the BAF
 
-                                
-                                
+                               
+
+
                                 error2 = error2 ** 0.5
-                                #print ('D', error2 , np.mean( (BAF1 - np.mean(BAF1)) ** 2 )  )
-                                #error1 = error1 * length0
-                                #error2 = error2 * length0
-
-                                #error1 = error1 + ((RDR_change ** 2) * (5 / length0))
-                                #error2 = error2 + ((BAF_change ** 2) * (5 / length0))
-
-
-                                #error2 = (error2 / (BAF_change ** 2)) + (5 / length0)
-                                #error2 = (error2 / length0) ** 0.5
-
-
-
-                        #error1 = error1 / length0
-                        #error2 = error2 / length0
-
-                        #length1 = length0 - 1
-                        #if a == 0:
-                        #    length1 += 2
-                        #if b == size1 - 1:
-                        #    length1 += 2
+                        
 
                         if type(BAF) != type(''):
                             error1 = 1/((1 / error1) + (1 / error2))
 
-                        if False:
-                            error1 = error1 + (0.1 * error2)
-                            error1 = error1 + 0.8
-
-                            error1 = error1 / length1
-
-                        #'''
-
-
-
-
-                        #error1_original = giveRegionError(RDR1, BAF1, a, b, size1)
-                        #error1 = error1_original
-                        #print (error1_original)
-                        #print (error1)
-                        #quit()
-
-
-
                         data1[count1, 0] = a
                         data1[count1, 1] = b
                         data1[count1, 2] = error1
@@ -941,7 +824,7 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
 
 
-        #print (count1)
+
         data1 = data1[:count1]
 
 
@@ -952,13 +835,6 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
         start1, end1 = int(data1[min1, 0]), int(data1[min1, 1])
 
-        #print (start1, end1)
-
-        #plt.plot(data1[:, 2])
-        #plt.show()
-        #plt.imshow(img)
-        #plt.show()
-        #quit()
 
         return start1, end1
 
@@ -989,7 +865,7 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
 
 
-                #print (RDR1.shape)
+
                 start2, end2 = findBestRegion(RDR1, BAF1)#, N)
 
                 regionDone[doneCount1, 0] = start1 + start2
@@ -1011,9 +887,7 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
         regionDone = regionDone[:doneCount1]
 
-        #print (regionDone)
-        #print (regionDone[np.argsort(regionDone[:, 0])])
-        #quit()
+        
 
         return regionDone
 
@@ -1021,19 +895,8 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
     def findAllRegions(RDR, BAF, start1, end1, chr):
 
         RDR_change = np.mean(np.abs(RDR[1:]-RDR[:-1]))
-        #if BAF == '':
-        #    BAF_change = ''
-        #else:
-        #    BAF_change = np.mean(np.abs(BAF[1:]-BAF[:-1]))
+        
 
-        #print (RDR_change, BAF_change)
-        #plt.plot(RDR)
-        #plt.plot(BAF)
-        #plt.show()
-
-        #N = 100
-
-        #regions = np.zeros((1000, 2), dtype=int)
         regions = np.zeros((10000, 2), dtype=int)
 
         count1 = 0
@@ -1046,32 +909,24 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
             regionDone = regionDone + start1[a]
             size1 = regionDone.shape[0]
 
-            #print (regionDone)
-            #quit()
-            #print (regionDone)
-            #print (size1)
+            
             regions[count1:count1+size1] = regionDone
-            #regions[count1] = regionDone
-            #print (regions)
-            #quit()
+            
             count1 += size1
-            #count1 += 1
+            
 
-        #quit()
 
-        #print (regions)
 
         regions = regions[:count1]
 
-        #print (regions)
+        
         sizes = regions[:, 1] - regions[:, 0]
         regions = regions[sizes > 6]
-        #quit()
+        
 
         regions = regions[np.argsort(regions[:, 0])]
 
-        #print (regions)
-        #quit()
+        
 
         return regions
 
@@ -1081,20 +936,9 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
     
 
 
-
-    #patientNum0 = '1'
-
-
-    #x = loadnpz('./data/input/S0.npz')
-    #x = loadnpz('./data/input/filtered_S' + patientNum0 + '.npz')
-
     RDR_all = loadnpz(RDR_file)
 
-    #N = 50
-    #RDR_all = RDR_all[:N * (RDR1.shape[0] // N)]
-    #RDR1 = RDR1.reshape((RDR1.shape[0] // N, N))
-    #RDR1 = np.mean(RDR1, axis=1)
-
+    
 
 
     if BAF_file == '':
@@ -1102,15 +946,10 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
     else:
         BAF_all = loadnpz(BAF_file)
 
-    #x = x[:500]
+    
 
     chr = loadnpz(chr_File)
 
-    #argGood = loadnpz('./data/input/argFilter_S0.npz')
-
-    #dataCall = loadnpz('./data/input/call_S0.npz')[argGood]
-    #Calls of Chisel
-    #dataCall = loadnpz('./data/input/callCNA_S' + patientNum0 + '.npz')
 
     _, start1 = np.unique(chr, return_index=True)
     end1 = np.concatenate((start1[1:], np.zeros(1) + chr.shape[0])).astype(int)
@@ -1125,44 +964,29 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
     #54
 
-    #for rand1 in np.random.randint(x.shape[0], size=100):
-    for a0 in range(0, RDR_all.shape[0]):
+    
+    for a0 in tqdm(range(0, RDR_all.shape[0])):
 
         #a = perm1[a0]
         a = a0
 
 
 
-        print (a, RDR_all.shape[0])
+        #print (a, RDR_all.shape[0])
 
-        #RDR = x[a, :, 0]
-        #BAF = x[a, :, 1]
+        
         RDR = RDR_all[a]
         if type(BAF_all) == type(''):
             BAF = ''
             BAF1 = ''
         else:
             BAF1 = BAF_all[a]
-            #BAF1 = np.min(np.array([BAF, 1 - BAF]), axis=0)
+            
             
 
         regions = findAllRegions(RDR, BAF1, start1, end1, chr)
 
-        #print (regions)
-        #quit()
-
-        print (regions.shape)
-
-        if False:
-            bool1 = np.zeros(RDR.shape[0])
-            for b in range(regions.shape[0]):
-                #print (regions[b, 0], regions[b, 1])
-                bool1[regions[b, 0]:regions[b, 1]] = 1 + (b%2)
-            plt.plot(bool1)
-            plt.plot(RDR)
-            #plt.plot(BAF1)
-            plt.show()
-
+        
 
 
         size1 = regions.shape[0]
@@ -1178,6 +1002,7 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
         count1 = count2
 
+
     regionList = regionList[:count1]
 
     np.savez_compressed(region_file, regionList)
@@ -1188,14 +1013,10 @@ def findRegions(RDR_file, BAF_file, chr_File, region_file):
 
 
 
-folder1 = 'DLP'
-#folder1 = '10x'
-#folder1 = 'ACT10x'
-#folder1 = 'TN3'
-chr_file = './data/' + folder1 + '/initial/chr_1M.npz'
-RDR_file = './data/' + folder1 + '/initial/RDR_1M.npz'
-region_file = './data/' + folder1 + '/binScale/regions.npz'
-HAP_file = './data/' + folder1 + '/initial/HAP_1M.npz'
+#chr_file = outFolder + '/initial/chr_1M.npz'
+#RDR_file = outFolder + '/initial/RDR_1M.npz'
+#region_file = outFolder + '/binScale/regions.npz'
+#HAP_file = outFolder + '/initial/HAP_1M.npz'
 #findRegions(RDR_file, HAP_file, chr_file, region_file)
 #quit()
 
@@ -1206,9 +1027,7 @@ HAP_file = './data/' + folder1 + '/initial/HAP_1M.npz'
 
 
 
-
-
-def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, dividerList_file, region_file, precise=True, naive=False):
+def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, dividerList_file, region_file, precise=True, naive=False, maxPloidy=10):
 
 
 
@@ -1273,6 +1092,9 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
 
                     std2 = error2_sq - ((mean2 ** 2) * weight_sq / (  weight_sum ** 2  ) )
 
+                    if std2 < 0:
+                        std2 = 0
+
                     
                     std2 = std2 + (0.25 / weight_sum ) #Adding the intrinsic noise in the BAF
 
@@ -1418,7 +1240,7 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
         return error1
 
 
-    def findBestDivider(means1, vars1, noises1, precise):
+    def findBestDivider(means1, vars1, noises1, precise, maxPloidy=10):
 
         #print ('noise')
         #print (np.sum(noises1[:, 0]))
@@ -1429,6 +1251,8 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
         #for b in range(50):
         #    divider = 0.2 + (b * 0.01)
 
+        
+
 
         Ncheck = 100
         tickSize = 0.02
@@ -1439,23 +1263,25 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
         for b in range(Ncheck):
             divider = 0.1 * np.exp(b * tickSize) #0.1
 
-            #for b in range(200):
-            #    divider = 0.1 * np.exp(b * 0.01)
+            if (1.0 / divider) < maxPloidy:
 
-            #divider = 0.15 * np.exp(b * 0.02)
+                #for b in range(200):
+                #    divider = 0.1 * np.exp(b * 0.01)
 
-            dividerList.append(divider)
+                #divider = 0.15 * np.exp(b * 0.02)
 
-            #print (divider)
-            #plt.plot(means1 / divider)
-            #plt.plot(np.floor(  (means1 / divider) + 0.5))
-            #plt.show()
+                dividerList.append(divider)
 
-            #print (means1.shape)
+                #print (divider)
+                #plt.plot(means1 / divider)
+                #plt.plot(np.floor(  (means1 / divider) + 0.5))
+                #plt.show()
 
-            error1 = checkDivider(means1, vars1, divider, noises1)
-            #BnumAll[b] = Bnum
-            errorList.append(error1)
+                #print (means1.shape)
+
+                error1 = checkDivider(means1, vars1, divider, noises1)
+                #BnumAll[b] = Bnum
+                errorList.append(error1)
 
         #plt.plot(dividerList, errorList)
         #plt.show()
@@ -1470,11 +1296,25 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
 
             Nprecise = 20
 
-            if argMin == Ncheck - 1:
-                argMin = Ncheck - 2
+            isLimited = False
+
+            if argMin >= dividerList.shape[0] - 1:
+                argMin = dividerList.shape[0] - 2
+                isLimited = True
 
             #print (argMin)
 
+            try:
+                dividerList[argMin + 1]
+            except:
+                print ("dividerList[argMin + 1] failed in scaler.py")
+                print ('argMin', argMin)
+                print ('dividerList', dividerList.shape)
+                print ('errorList', errorList.shape)
+                print ('isLimited', isLimited)
+
+
+
             dividerBefore = dividerList[argMin - 1]
             dividerAfter = dividerList[argMin + 1]
             dividerTick = (dividerAfter - dividerBefore) / float(Nprecise)
@@ -1703,14 +1543,10 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
 
 
     #for rand1 in np.random.randint(x.shape[0], size=100):
-    for a in range(0, RDR_all.shape[0]):
+    for a in tqdm(range(RDR_all.shape[0])):
 
         #a = perm1[a]
 
-        print (a, RDR_all.shape[0])
-
-        #RDR = x[a, :, 0]
-        #BAF = x[a, :, 1]
         RDR = RDR_all[a]
         if type(HAP_all) == type(''):
             HAP = ''
@@ -1725,10 +1561,7 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
         count2 = count1
         while regionList[count2 % regionList.shape[0], 0] == a:
             count2 += 1
-        #count2 += 1
-
-        #print (regionList[count1:count2])
-        #quit()
+        
 
         regions = regionList[count1:count2, 1:]
         count1 = count2
@@ -1751,19 +1584,13 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
 
             noises1 = ''
 
-            divideGood, dividerList, errorList = findBestDivider(means1, vars1, noises1, precise)
+            divideGood, dividerList, errorList = findBestDivider(means1, vars1, noises1, precise, maxPloidy=maxPloidy)
 
 
-            #print ('done1')
-            #quit()
             
 
-
-            #plt.plot(errorList / (chr.shape[0] * 2))
-            #plt.show()
-
-
-            if False:#a >= 100:#abs(divideGood - 0.27) > 0.04:#a > 80:#a in [0, 18, 66]:#[0, 18]:
+            #The code within the False if statement is for debugging during developement 
+            if False:
                 print (divideGood)
                 meansPlot = np.zeros(RDR.shape[0])
                 bool1 = np.zeros(RDR.shape[0])
@@ -1778,34 +1605,12 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
                 #plt.plot(RDR)
                 plt.plot(RDR / divideGood)
                 plt.plot(meansPlot)
-
-
                 #plt.plot(BAF1)
                 plt.show()
 
-            if False:#a == 18:
-
-                BAF = HAP[:, 0] / (np.sum(HAP, axis=1 ) + 1e-5)
-
-                BAFplot = (BAF-1)*2
-                BAFplot[ np.sum(HAP, axis=1) < 10 ] = BAFplot[ np.sum(HAP, axis=1) < 10 ] / 0
-
-                plt.plot(np.zeros(RDR.shape[0])-2, c='grey')
-                plt.plot(np.zeros(RDR.shape[0])-1, c='grey')
-                plt.plot(np.zeros(RDR.shape[0]), c='grey')
-                plt.plot(np.zeros(RDR.shape[0])+1, c='grey')
-                plt.plot(np.zeros(RDR.shape[0])+2, c='grey')
-                plt.plot(np.zeros(RDR.shape[0])+3, c='grey')
-                plt.plot(np.zeros(RDR.shape[0])+4, c='grey')
-                plt.plot(np.zeros(RDR.shape[0])+5, c='grey')
-                plt.plot(RDR / divideGood)
-                plt.plot(  BAFplot   )
-                plt.show()
-            #quit()
 
         dividerNums.append(divideGood)
 
-        #quit()
 
         if not created1:
             divideAll = np.zeros((RDR_all.shape[0], dividerList.shape[0]))
@@ -1822,7 +1627,7 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
 
     #quit()
 
-    print ("done1")
+    #print ("done1")
 
     np.savez_compressed(divider_file, dividerNums)
     np.savez_compressed(error_file, divideError)
@@ -1833,29 +1638,6 @@ def findDividers(RDR_file, HAP_file, chr_File, divider_file, error_file, divider
 
 
 
-
-#RDR_file = './data/DLP/filtered_RDR.npz'
-#chr_File = './data/DLP/chr.npz'
-#region_file = './data/DLP/regions.npz'
-#divider_file = './data/DLP/dividers.npz'
-#error_file = './data/DLP/dividerError.npz'
-#dividerList_file = './data/DLP/dividerAll.npz'
-#BAF_file = ''
-
-
-#patientNum0 = '1'
-#RDR_file = './data/input/filtered_RDR_S' + patientNum0 + '.npz'
-#BAF_file = './data/input/filtered_BAF_S' + patientNum0 + '.npz'
-#chr_File = './data/input/chr_S' + patientNum0 + '.npz'
-#divider_file = './data/inputResults/S' + patientNum0 + '_dividers.npz'
-#error_file = './data/inputResults/S' + patientNum0 + '_dividerError.npz'
-#dividerList_file = './data/inputResults/S' + patientNum0 + '_dividerAll.npz'
-#region_file = './data/inputResults/S' + patientNum0 + '_regions.npz'
-
-
-
-
-
 #folder1 = 'DLP'
 folder1 = '10x'
 #folder1 = 'ACT10x'
@@ -1875,15 +1657,6 @@ HAP_file = './data/' + folder1 + '/initial/HAP_1M.npz'
 
 
 
-#RDR = loadnpz(RDR_file)
-#mean1 = np.mean(RDR[:, 50:200], axis=1)
-#RDR = RDR / mean1.reshape((-1, 1))
-#import seaborn as sns
-#plt.plot(RDR[:10].T)
-#plt.show()
-#quit()
-
-
 def newFindDividers(bins_file, RDR_file, noise_file, BAF_file, BAF_noise_file, divider_file, error_file, dividerList_file):
 
  
@@ -2159,9 +1932,14 @@ def newFindDividers(bins_file, RDR_file, noise_file, BAF_file, BAF_noise_file, d
 
 
 
+
+
+
+
+
+
 def findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, divider_file, error_file, dividerList_file, initialCNA_file, initialUniqueCNA_file, initialUniqueIndex_file):
 
-    
 
     def findCurrentCNA(RDR, noise, HAP, BAF_noise):
 
@@ -2215,14 +1993,18 @@ def findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, div
 
                 #BAF_now_adjusted = (BAF_now * 0.9) + 0.05
                 #BAF_now_adjusted = (BAF_now * 0.95) + 0.025
-                BAF_now_adjusted = tweakBAF(BAF_now)
+                #BAF_now_adjusted = (BAF_now * 0.99) + 0.005
+                BAF_now_adjusted = (BAF_now * 0.999) + 0.0005
+                #BAF_now_adjusted = tweakBAF(BAF_now)
 
 
                 HAP_now = HAP[argValid] 
+                #HAP_mod_now = HAP_mod[argValid] 
 
-                #BAF_error = (np.log(BAF_now_adjusted) * HAP_now[:, 1]) + (np.log(1 - BAF_now_adjusted) * HAP_now[:, 0] )
-                #BAF_error = BAF_error * -1
-
+                #if False:
+                #    BAF_error = (np.log(BAF_now_adjusted) * HAP_mod_now[:, 1]) + (np.log(1 - BAF_now_adjusted) * HAP_mod_now[:, 0] )
+                #    BAF_error = BAF_error * -1
+                #else:
                 BAF_measure = HAP_now[:, 1] / (np.sum(HAP_now, axis=1) + 1e-5)
                 BAF_error = ((BAF_measure - BAF_now) / BAF_noise[argValid]) ** 2
                 
@@ -2354,6 +2136,7 @@ def findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, div
             #quit()
 
             BAF = BAF_all[a]
+            #HAP_mod = HAP_mod_all[a]
             noise = noise_all[a]
             BAF_noise = BAF_noise_all[a]
             #BAF = np.min(np.array([BAF, 1-BAF]), axis=0)
@@ -2362,6 +2145,7 @@ def findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, div
             #RDR, BAF = RDR_all[a], BAF_all[a]
             RDR = RDR / dividerNums[a]
 
+            #CNA = findCurrentCNA(RDR, noise, BAF, HAP_mod, BAF_noise)
             CNA = findCurrentCNA(RDR, noise, BAF, BAF_noise)
 
 
@@ -2401,6 +2185,7 @@ def findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, div
     else:
         BAF_all = loadnpz(BAF_file)[:dividerNums.shape[0]]
         BAF_noise_all = loadnpz(BAF_noise_file)[:dividerNums.shape[0]] + 1e-5
+        #HAP_mod = loadnpz(HAP_mod_file)
     
 
     #chr = loadnpz('./data/input/chr_S' + patientNum0 + '.npz')
@@ -2437,15 +2222,13 @@ def findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, div
     subsetSingle = np.argwhere(indexCounts==1)[:, 0]
     
 
-    #print (indexFirst.shape)
-    #print (argList.shape)
-    #quit()
 
     
     if type(BAF_all) == type(''):
         CNAfull = np.round(RDR_all[argList] / dividerNums.reshape((-1, 1)) ).astype(int)
     else:
-        CNAfull = doInitialPart(dividerNums, RDR_all[argList], noise[argList], BAF_all[argList], BAF_noise_all[argList], chr)
+        #CNAfull = doInitialPart(dividerNums, RDR_all[argList], noise[argList], BAF_all[argList], HAP_mod[argList],  BAF_noise_all[argList], chr)
+        CNAfull = doInitialPart(dividerNums, RDR_all[argList], noise[argList], BAF_all[argList],  BAF_noise_all[argList], chr)
         
 
     
@@ -2459,12 +2242,23 @@ def findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, div
     CNAfull = CNAfull[index1]
     if type(BAF_all) != type(''):
         CNAfull = CNAfull.reshape((CNAfull.shape[0], CNAfull.shape[1]//2, 2))
-
-    np.savez_compressed(initialUniqueCNA_file, CNAfull)
-
     relevantIndex = inverse1[indexFirst]
 
-    np.savez_compressed(initialUniqueIndex_file, relevantIndex)
+
+    #plt.imshow(np.sum(CNAfull, axis=2))
+    #plt.show()
+
+    CNAfull_total = np.sum(CNAfull, axis=2)
+    highNoise = np.sum(np.abs( CNAfull_total[:, 1:] - CNAfull_total[:, :-1] ), axis=1)
+    #print (highNoise.shape)
+    #print (CNAfull.shape)
+    #print (relevantIndex.shape)
+    argGood = np.argwhere(highNoise <= 500)[:, 0]
+    CNAfull = CNAfull[argGood]
+    #relevantIndex = relevantIndex[argGood]
+
+    np.savez_compressed(initialUniqueCNA_file, CNAfull)
+    #np.savez_compressed(initialUniqueIndex_file, relevantIndex)
 
 
 
@@ -2472,13 +2266,6 @@ def findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, div
 
 
 
-#folder1 = 'ACT/P1'
-#folder1 = 'DLP'
-#folder1 = '10x'
-#folder1 = 'ACT10x'
-#folder1 = 'TN3'
-
-
 RDR_file = './data/' + folder1 + '/binScale/filtered_RDR_avg.npz'
 noise_file = './data/' + folder1 + '/binScale/filtered_RDR_noise.npz'
 chr_file = './data/' + folder1 + '/binScale/chr_avg.npz'
@@ -2494,11 +2281,6 @@ initialUniqueIndex_file = './data/' + folder1 + '/binScale/initialIndex.npz'
 #findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, divider_file, error_file, dividerList_file, initialCNA_file, initialUniqueCNA_file, initialUniqueIndex_file)
 #quit()
 
-#RDR = loadnpz(RDR_file)
-#CNA = loadnpz(initialCNA_file)
-#print (CNA.shape)
-#print (RDR.shape)
-#quit()
 
 
 def saveReformatCSV(outLoc, isNaive=False):
@@ -2517,16 +2299,8 @@ def saveReformatCSV(outLoc, isNaive=False):
     _, chrStarts = np.unique(chrAll, return_index=True)
     goodSubset = goodSubset - chrStarts[chr1]
 
-    #print (cellNames)
-    #quit()
-
     
 
-    #import matplotlib.pyplot as plt
-    #plt.plot(goodSubset)
-    #plt.savefig('./images/temp.png')
-    #quit()
-
     _, index_start = np.unique(bins, return_index=True)
     _, index_end = np.unique(bins[-1::-1], return_index=True)
     index_end = bins.shape[0] - 1 - index_end
@@ -2574,17 +2348,16 @@ def saveReformatCSV(outLoc, isNaive=False):
 
 
 
-
-
-#outLoc = './data/newTN3'
-#saveReformatCSV(outLoc)
-#quit()
-
-
-
-
 def scalorRunBins(outLoc):
+
     
+    numSteps = '9'
+    stepName = 8
+
+
+    stepName += 1
+    stepString = str(stepName) + '/' + numSteps
+    print ('Data processing — Step ' + stepString + ': Creating segements... ', end='')
 
     chr_file = outLoc + '/initial/chr_100k.npz'
     RDR_file = outLoc + '/initial/RDR_100k.npz'
@@ -2595,7 +2368,6 @@ def scalorRunBins(outLoc):
     findVariableBins(RDR_file, bins_file, chr_file, totalRead_file, doBAF, BAF_file=BAF_file)
 
     
-
     bins_file = outLoc + '/binScale/bins.npz'
     chr_file = outLoc + '/initial/chr_100k.npz'
     RDR_file = outLoc + '/initial/RDR_100k.npz'
@@ -2607,24 +2379,27 @@ def scalorRunBins(outLoc):
     BAF_file2 = outLoc + '/binScale/filtered_HAP_avg.npz'
     BAF_noise_file = outLoc + '/binScale/BAF_noise.npz'
     applyVariableBins(RDR_file, bins_file, chr_file, RDR_file2, noise_file, chr_file2, doBAF, BAF_file=BAF_file, BAF_file2=BAF_file2, BAF_noise_file=BAF_noise_file)
+    print ("Done")
 
 
 
 
 
-def runNaiveCopy(outLoc):
+def runNaiveCopy(outLoc, maxPloidy=10):
     
 
-    
 
+    print ('NaiveCopy — Step 1/3: Finding low variance regions... ')
     chr_file = outLoc + '/initial/chr_1M.npz'
     RDR_file = outLoc + '/initial/RDR_1M.npz'
     region_file = outLoc + '/binScale/regions.npz'
     HAP_file = outLoc + '/initial/HAP_1M.npz'
     findRegions(RDR_file, HAP_file, chr_file, region_file)
+    #print ('Done')
 
 
 
+    print ('NaiveCopy — Step 2/3: Finding cell specific scaling factors... ')
     RDR_file = outLoc + '/initial/RDR_1M.npz'
     chr_file = outLoc + '/initial/chr_1M.npz'
     region_file = outLoc + '/binScale/regions.npz'
@@ -2632,10 +2407,11 @@ def runNaiveCopy(outLoc):
     error_file = outLoc + '/binScale/dividerError.npz'
     dividerList_file = outLoc + '/binScale/dividerAll.npz'
     HAP_file = outLoc + '/initial/HAP_1M.npz'
-    findDividers(RDR_file, HAP_file, chr_file, divider_file, error_file, dividerList_file, region_file)
-
+    findDividers(RDR_file, HAP_file, chr_file, divider_file, error_file, dividerList_file, region_file, maxPloidy=maxPloidy)
+    #print ('Done')
 
 
+    print ('NaiveCopy — Step 3/3: Estimating copy number profiles... ')
     RDR_file = outLoc + '/binScale/filtered_RDR_avg.npz'
     noise_file = outLoc + '/binScale/filtered_RDR_noise.npz'
     chr_file = outLoc + '/binScale/chr_avg.npz'
@@ -2647,21 +2423,24 @@ def runNaiveCopy(outLoc):
     initialCNA_file = outLoc + '/binScale/initialCNA.npz'
     initialUniqueCNA_file = outLoc + '/binScale/initialUniqueCNA.npz'
     initialUniqueIndex_file = outLoc + '/binScale/initialIndex.npz'
+    #HAP_mod_file = outLoc + '/binScale/HAP_mod.npz'
     findInitialCNA(RDR_file, noise_file, BAF_file, BAF_noise_file, chr_file, divider_file, error_file, dividerList_file, initialCNA_file, initialUniqueCNA_file, initialUniqueIndex_file)
     
+    
 
     saveReformatCSV(outLoc, isNaive=True)
+    print ('Done')
 
 
-def scalorRunAll(outLoc):
+def scalorRunAll(outLoc, maxPloidy=10):
     scalorRunBins(outLoc)
-    runNaiveCopy(outLoc)
+    runNaiveCopy(outLoc, maxPloidy=maxPloidy)
 
 
 #outLoc = './data/newTN3'
 #scalorRunAll(outLoc)
 
-#outLoc = '/Users/stefanivanovic/Desktop/Coding/Bio/CNA/data/TN3'
+
 #saveReformatCSV(outLoc, isNaive=True)
 #saveReformatCSV(outLoc, isNaive=False)
 #quit()
\ No newline at end of file
